%  PhyloPattern
%  Copyright (C) 2005-2009 Philippe Gouret
%  This library is free software; you can redistribute it and/or
%  modify it under the terms of the GNU General Public
%  License as published by the Free Software Foundation; either
%  version 2 of the License, or (at your option) any later version.
%  This library is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%  General Public License for more details.
% 
%  You should have received a copy of the GNU General Public
%  License along with this library; if not, write to the
%  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
%  Boston, MA  02111-1307, USA. The text ol license can be also found 
%  at http://www.gnu.org/copyleft/gpl.html
%

%###############################################################################
%
%			tools for PhyloPattern
%
%					Philippe Gouret 2006-2009
%
%###############################################################################


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                               miscellaneous tools
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% bagof always successfull => empty list if no solution
bagof_always_successfull(X, G, L) :- findall(X, G, L), !.
bagof_always_successfull(_, _, []).

% convert atom to number
atom2number(Atom, Number) :-
  atom_chars(Atom, Chars),
  number_chars(Number, Chars).
number2atom(Number, Atom) :-
  number_chars(Number, Chars),
  atom_chars(Atom, Chars).

% list's cardinal
card([], 0).
card([_|L], X) :-
  card(L, Y),
  is(X, Y + 1).
    
% concatenation of two lists
concat(L, [], L) :- !.
concat([], L, L) :- !.
concat([X|L], M, [X|N]) :-
  concat(L, M, N).

% console writing tools
println(X) :- write(X), write('\n'), flush_output.
print(X) :-  write(X), flush_output.
qprintln(X) :- write_term(X, [quoted(true)]), write('\n'), flush_output.
qprint(X) :-  write_term(X, [quoted(true)]), flush_output.
qprintln_list([]).
qprintln_list([One|Left]) :-
	qprintln(One),
	qprintln_list(Left).

% success if bounded X and Y are not unifiable
diff(X, X) :- !, fail.
diff(_, _).

% success if X is contained in list
element(X, [X|_]).
element(X, [Y|L]) :- element(X, L), diff(X, Y).

% eliminate all occurance of an element in a list
without(_, [], []).
without(X, [X|L], M) :- without(X, L, M).
without(X, [Y|L], [Y|M]) :- diff(X,Y), without(X, L, M).

% eliminate multiple occurances of elements in a list
only_once([], Result, Result).
only_once([X|L], Built, Result) :-
  element(X, Built), !,
  only_once(L, Built, Result).
only_once([X|L], Built, Result) :-
  only_once(L, [X|Built], Result).

% give two lists intersection
lists_intersection(_, [], []) :- !.
lists_intersection([], _, []) :- !.
lists_intersection([X|L], M, [X|I]) :-
  element(X, M),
  lists_intersection(L, M, I), !.
lists_intersection([_|L], M, I) :-
  lists_intersection(L, M, I).

% one success, no backtracking
one_success(Goal) :- Goal, !.

% test negation of fact 
not(X) :- X, !, fail.
not(_).

% execute a goal for each element of a list
for_each([], _).
for_each([X|L], Goal) :-
	concat(Goal, [X], GoalWithParams),
	Exec =.. GoalWithParams,
	Exec,
	for_each(L, Goal).
	
% verify that there is one element in the list that verify the goal
exists_one([], _, _) :- fail.
exists_one([X|L], Goal, X) :-
	concat(Goal, [X], GoalWithParams),
	Exec =.. GoalWithParams,
	Exec, !.
exists_one([_|L], Goal, One) :- exists_one(L, Goal, One).

% test if two lists are equivalent (same elements but not always in the same order)
lists_equivalent(L, L) :- !.
lists_equivalent(L1, L2) :- lists_intersection(L1, L2, L3), card(L1, Card), card(L2, Card), card(L3, Card).

% transform a list of lists in a list of all elements in the lists
lists_fusion([], Result, Result).
lists_fusion([List|Lists], Partial, Result) :-
	concat(List, Partial, NewPartial),
	lists_fusion(Lists, NewPartial, Result).

% eliminate element at specified position
% +list to cut, +initial count, +position to eliminate, -eliminated element
% -list before element, -list after elimination
eliminate_element_at_pos([X|After], Count, Count, X, Before, Result) :-
  concat(Before, After, Result), !.
eliminate_element_at_pos([X|L], Count, Pos, Elem, Before, Result) :-
  Count < Pos,
  is(NewCount, Count + 1),
  eliminate_element_at_pos(L, NewCount, Pos, Elem, [X|Before], Result).
	  
% order elements around a pivot value
% +list to be ordonated, +predicate used to compare with pivot, +pivot value
% -all elements before (given by comparer) pivot, -all elements after pivot
order([], _, _, [], []) :- !.
order([X|L], Comparer, Pivot, SubList1, [X|SL2]) :-
  Compare =.. [Comparer, X, Pivot],
  Compare,
  order(L, Comparer, Pivot, SubList1, SL2), !.
order([X|L], Comparer, Pivot, [X|SL1], SubList2) :-
  order(L, Comparer, Pivot, SL1, SubList2).

% quick sort (element respecting Comparer condition are placed at the end)
quick_sort_partition(List, Comparer, SubList1, SubList2, Pivot) :-
  card(List, Card),
  is(Middle, Card // 2),
  eliminate_element_at_pos(List, 0, Middle, Pivot, [], NewList),
  order(NewList, Comparer, Pivot, SubList1, SubList2).
% +list to sort, +predicate used to compare, -sorted list
quick_sort([], _, []) :- !.
quick_sort([X], _, [X]) :- !.
quick_sort(List, Comparer, Result) :-
  quick_sort_partition(List, Comparer, SL1, SL2, Pivot),
  quick_sort(SL1, Comparer, L1),
  quick_sort(SL2, Comparer, L2),
  concat(L1, [Pivot], LL),
  concat(LL, L2, Result).


%###############################################################################













